export VARNAME_CI_SECRET="PYUTILS_CI_SECRET"
export VARNAME_TWINE_PASSWORD="PYUTILS_PYPI_MASTER_TOKEN"
export VARNAME_TEST_TWINE_PASSWORD="PYUTILS_TEST_PYPI_MASTER_TOKEN"
export VARNAME_TWINE_USERNAME="PYUTILS_PYPI_MASTER_TOKEN_USERNAME"
export VARNAME_TEST_TWINE_USERNAME="PYUTILS_TEST_PYPI_MASTER_TOKEN_USERNAME"
export GPG_IDENTIFIER="=PyUtils-CI <openpyutils@gmail.com>"
